from .request_dtr import request_dtr
from .helpers import specify_cardinality
from .helpers import format_string


def extract_epic(datatype_id):
    """
    Extract ePIC datatype information

    :param datatype_id: an identifier of an ePIC datatype
    :return: a dictionary with structured information of the ePIC datatype
    """
    extract_all = {}

    def extractor_function(datatype_id):
        info = request_dtr(datatype_id + "?locatt=view:json")
        schema_dict = {
            "dt_name": format_string(info["name"]),
            "dt_id": info["Identifier"].split("/", 4)[1],
            "dt_class": info["Schema"]["Type"]}
        extracted = [[schema_dict]]
        all_props = []
        for prop in info["Schema"].get("Properties", []):
            if "Type" not in prop:
                specific_prop_dict = {
                    "dtp_name": format_string(prop["Property"]),
                    "dtp_id": info["Identifier"] + "#" + format_string(prop["Property"]),
                    "dtp_card_min": None,
                    "dtp_card_max": None,
                    "dtp_value_type": prop["Value"]}
            else:
                card = specify_cardinality(prop["Properties"]["Cardinality"])
                specific_prop_dict = {
                    "dtp_name": format_string(prop["Name"]),
                    "dtp_id": info["Identifier"] + "#" + format_string(prop["Name"]),
                    "dtp_card_min": card["min"],
                    "dtp_card_max": card["max"],
                    "dtp_value_type": prop["Type"]}
                if prop["Type"] != "":
                    extractor_function("https://doi.org/" + prop["Type"])
            all_props.append(specific_prop_dict)
        extracted.append(all_props)
        extract_all[schema_dict["dt_name"]] = list(extracted)
    extractor_function(datatype_id)
    return extract_all



from .request_dtr import request_dtr
from .helpers import format_string


def extract_orkg(datatype_id):
    """
    Extract ORKG template information

    :param datatype_id: an identifier of an ORKG template
    :return: a dictionary with structured information of the ORKG template
    """
    part = datatype_id.split("/", 4)
    orkg_hostname = part[0] + "//" + part[2]
    resource_id = part[4]
    extract_all = {}

    def extractor_function(resource_id):
        info = request_dtr(orkg_hostname + "/api/templates/" + resource_id)
        schema_dict = {
            "dt_name": format_string(info["label"]),
            "dt_id": info["id"],
            "dt_class": info["target_class"]["id"]}
        extracted = [[schema_dict]]
        all_props = []
        for prop in info.get("properties", []):
            specific_prop_dict = {
                "dtp_name": format_string(prop["path"]["label"]),
                "dtp_id": prop["path"]["id"],
                "dtp_card_min": prop["min_count"],
                "dtp_card_max": prop["max_count"]
            }
            if "class" not in prop:
                if "datatype" not in prop:
                    specific_prop_dict["dtp_value_type"] = ""
                else:
                    specific_prop_dict["dtp_value_type"] = prop["datatype"]["id"]
            else:
                specific_prop_dict["dtp_value_type"] = prop["class"]["id"]
                info_n = request_dtr(
                    orkg_hostname + "/api/templates/?target_class=" + prop["class"]["id"])
                if len(info_n["content"]) > 0:
                    nested_id = info_n["content"][0]["id"]
                    nested_name = info_n["content"][0]["label"]
                    if nested_name not in extract_all:
                        extractor_function(nested_id)
            all_props.append(specific_prop_dict)
        all_props.append({
            "dtp_name": "label",
            "dtp_id": "label",
            "dtp_card_min": 0,
            "dtp_card_max": 1,
            "dtp_value_type": "string"})
        extracted.append(all_props)
        extract_all[schema_dict["dt_name"]] = list(extracted)
    extractor_function(resource_id)
    return extract_all



import unittest
from unittest.mock import patch
from helpers_mock.mocking import mocked_request_orkg
from dtreg.extract_orkg import extract_orkg


@patch("dtreg.extract_orkg.request_dtr", mocked_request_orkg)
class TestExtractOrkg(unittest.TestCase):

    def test_extract_orkg(self):
        result = extract_orkg("https://orkg.org/template/R758316")
        expected = {'dtreg_test_template2': [[{'dt_name': 'dtreg_test_template2',
                                               'dt_id': 'R758316',
                                               'dt_class': 'C102007'}],
                                             [{'dtp_name': 'property3',
                                               'dtp_id': 'P160024',
                                               'dtp_card_min': 0,
                                               'dtp_card_max': None,
                                               'dtp_value_type': 'Integer'},
                                              {'dtp_name': 'label',
                                                 'dtp_id': 'label',
                                                 'dtp_card_min': 0,
                                                 'dtp_card_max': 1,
                                                 'dtp_value_type': 'string'}]]}
        self.assertEqual(result, expected)

    def test_extract_orkg_props(self):
        schema = extract_orkg("https://orkg.org/template/R758316")
        values = schema["dtreg_test_template2"][1][0].values()
        expected = "dict_values(['property3', 'P160024', 0, None, 'Integer'])"
        self.assertEqual(str(values), expected)

    def test_extract_orkg_nested(self):
        schema = extract_orkg("https://orkg.org/template/R758315")
        expected = "dict_keys(['dtreg_test_template2', 'dtreg_test_template1'])"
        self.assertEqual(str(schema.keys()), expected)


if __name__ == '__main__':
    unittest.main()



import unittest
from dtreg.from_static import from_static


class TestFromStatic(unittest.TestCase):

    def test_no_static(self):
        no_static = from_static("https://doi.org/21.T11969/111")
        self.assertEqual(no_static, None)

    def test_static(self):
        templ = from_static("https://doi.org/21.T11969/3df63b7acb0522da685d")
        expected = {'string': [[{'dt_name': 'string',
                                 'dt_id': '3df63b7acb0522da685d',
                                 'dt_class': 'String'}],
                               []]}
        self.assertEqual(templ, expected)


if __name__ == '__main__':
    unittest.main()



import unittest
from unittest.mock import patch
from helpers_mock.mocking import mocked_request_orkg
import pandas as pd
from dtreg.load_datatype import load_datatype
from dtreg.dtr_interface import Epic
from dtreg.helpers import generate_uid
from dtreg.to_jsonld import differ_input, to_jsonld


class TestJsonLd(unittest.TestCase):

    maxDiff = None

    def test_input_numeric(self):
        self.assertEqual(differ_input(3), 3)

    def test_input_df(self):
        df = pd.DataFrame({'A': [1, 2], 'B': [3, 4]})
        new_uid = generate_uid()
        new_constants = Epic().add_df_constants()
        with patch('dtreg.to_jsonld.constants', new_constants):
            with patch('dtreg.to_jsonld.uid', new_uid):
                result = differ_input(df)
                self.assertEqual(result["tab_label"], "Table")

    def test_input_named_df(self):
        df = pd.DataFrame({'A': [1, 2], 'B': [3, 4]})
        df.name = "my_results"
        new_uid = generate_uid()
        new_constants = Epic().add_df_constants()
        with patch('dtreg.to_jsonld.constants', new_constants):
            with patch('dtreg.to_jsonld.uid', new_uid):
                result = differ_input(df)
                self.assertEqual(result["tab_label"], "my_results")

    def test_missing_df(self):
        df = pd.DataFrame({'A': [None, 2], 'B': [3, 4]})
        new_uid = generate_uid()
        new_constants = Epic().add_df_constants()
        with patch('dtreg.to_jsonld.constants', new_constants):
            with patch('dtreg.to_jsonld.uid', new_uid):
                result = differ_input(df)
                self.assertEqual(result["rows"][0]["cells"][0]["value"], None)

    def test_no_function(self):
        def abc(x):
            return x+1
        dt = load_datatype("https://doi.org/21.T11969/aff130c76e68ead3862e")
        instance = dt.data_item(source_table=abc)
        to_jsonld(instance)
        self.assertRaisesRegex(
            ValueError, "SystemExit: Input in source_table should not be a function")

    def test_jsonld_epic_nested(self):
        dt = load_datatype("https://doi.org/21.T11969/aff130c76e68ead3862e")
        url = dt.url()
        instance = dt.data_item(has_expression=url)
        result = to_jsonld(instance)
        expected = ('{\n'
                    '  "@id": "_:n1",\n'
                    '  "@type": "doi:aff130c76e68ead3862e",\n'
                    '  "doi:aff130c76e68ead3862e#has_expression": {\n'
                    '    "@id": "_:n2",\n'
                    '    "@type": "doi:e0efc41346cda4ba84ca"\n'
                    '  },\n'
                    '  "@context": {\n'
                    '    "doi": "https://doi.org/21.T11969/",\n'
                    '    "columns": "https://doi.org/21.T11969/0424f6e7026fa4bc2c4a#columns",\n'
                    '    "col_number": "https://doi.org/21.T11969/65ba00e95e60fb8971e6#number",\n'
                    '    "col_titles": "https://doi.org/21.T11969/65ba00e95e60fb8971e6#titles",\n'
                    '    "rows": "https://doi.org/21.T11969/0424f6e7026fa4bc2c4a#rows",\n'
                    '    "row_number": "https://doi.org/21.T11969/9bf7a8e8909bfd491b38#number",\n'
                    '    "row_titles": "https://doi.org/21.T11969/9bf7a8e8909bfd491b38#titles",\n'
                    '    "cells": "https://doi.org/21.T11969/9bf7a8e8909bfd491b38#cells",\n'
                    '    "column": "https://doi.org/21.T11969/4607bc7c42ac8db29bfc#column",\n'
                    '    "value": "https://doi.org/21.T11969/4607bc7c42ac8db29bfc#value",\n'
                    '    "tab_label": "https://doi.org/21.T11969/0424f6e7026fa4bc2c4a#label"\n'
                    '  }\n'
                    '}')
        self.assertEqual(result, expected)

    def test_jsonld_epic_list(self):
        dt = load_datatype("https://doi.org/21.T11969/aff130c76e68ead3862e")
        url_1 = dt.url()
        url_2 = dt.url()
        instance = dt.data_item(has_expression=[url_1, url_2])
        result = to_jsonld(instance)
        expected = ('{\n'
                    '  "@id": "_:n1",\n'
                    '  "@type": "doi:aff130c76e68ead3862e",\n'
                    '  "doi:aff130c76e68ead3862e#has_expression": [\n'
                    '    {\n'
                    '      "@id": "_:n2",\n'
                    '      "@type": "doi:e0efc41346cda4ba84ca"\n'
                    '    },\n'
                    '    {\n'
                    '      "@id": "_:n3",\n'
                    '      "@type": "doi:e0efc41346cda4ba84ca"\n'
                    '    }\n'
                    '  ],\n'
                    '  "@context": {\n'
                    '    "doi": "https://doi.org/21.T11969/",\n'
                    '    "columns": "https://doi.org/21.T11969/0424f6e7026fa4bc2c4a#columns",\n'
                    '    "col_number": "https://doi.org/21.T11969/65ba00e95e60fb8971e6#number",\n'
                    '    "col_titles": "https://doi.org/21.T11969/65ba00e95e60fb8971e6#titles",\n'
                    '    "rows": "https://doi.org/21.T11969/0424f6e7026fa4bc2c4a#rows",\n'
                    '    "row_number": "https://doi.org/21.T11969/9bf7a8e8909bfd491b38#number",\n'
                    '    "row_titles": "https://doi.org/21.T11969/9bf7a8e8909bfd491b38#titles",\n'
                    '    "cells": "https://doi.org/21.T11969/9bf7a8e8909bfd491b38#cells",\n'
                    '    "column": "https://doi.org/21.T11969/4607bc7c42ac8db29bfc#column",\n'
                    '    "value": "https://doi.org/21.T11969/4607bc7c42ac8db29bfc#value",\n'
                    '    "tab_label": "https://doi.org/21.T11969/0424f6e7026fa4bc2c4a#label"\n'
                    '  }\n'
                    '}')
        self.assertEqual(result, expected)

    @patch("dtreg.extract_orkg.request_dtr", mocked_request_orkg)
    def test_jsonld_orkg(self):
        dt = load_datatype("https://orkg.org/template/R758316")
        instance = dt.dtreg_test_template2(label="test")
        result = to_jsonld(instance)
        expected = ('{\n'
                    '  "@id": "_:n1",\n'
                    '  "@type": "orkgr:R758316",\n'
                    '  "label": "test",\n'
                    '  "@context": {\n'
                    '    "orkgc": "https://orkg.org/class/",\n'
                    '    "orkgr": "https://orkg.org/resource/",\n'
                    '    "orkgp": "https://orkg.org/property/",\n'
                    '    "columns": "https://orkg.org/property/CSVW_Columns",\n'
                    '    "col_number": "https://orkg.org/property/CSVW_Number",\n'
                    '    "col_titles": "https://orkg.org/property/CSVW_Titles",\n'
                    '    "rows": "https://orkg.org/property/CSVW_Rows",\n'
                    '    "row_number": "https://orkg.org/property/CSVW_Number",\n'
                    '    "row_titles": "https://orkg.org/property/CSVW_Titles",\n'
                    '    "cells": "https://orkg.org/property/CSVW_Cells",\n'
                    '    "column": "https://orkg.org/property/CSVW_Column",\n'
                    '    "value": "https://orkg.org/property/CSVW_Value",\n'
                    '    "label": "http://www.w3.org/2000/01/rdf-schema#label",\n'
                    '    "tab_label": "http://www.w3.org/2000/01/rdf-schema#label"\n'
                    '  }\n'
                    '}')
        self.assertEqual(result, expected)


if __name__ == '__main__':
    unittest.main()



from typing import Protocol
from .extract_epic import extract_epic
from .extract_orkg import extract_orkg
from .from_static import from_static


def select_dtr(datatype_id):
    """
    Select a dtr-related class based on a schema identifier

    :param datatype_id: a schema identifier
    :return: a class related to the datatype registry
    """
    selected_class = None
    if datatype_id.split("/", 4)[3] == '21.T11969':
        selected_class = Epic
    elif "orkg.org" in datatype_id.split("/", 4)[2]:
        selected_class = Orkg
    else:
        print("Please check whether the schema belongs to the ePIC or the ORKG dtr")
    return selected_class


class DataTypeReg(Protocol):
    """
    Interface for a class representing a datatype registry
    """

    def get_schema_info(self, datatype_id):
        """
        Obtain information from a schema

        :param datatype_id: a schema identifier
        :return: not implemented, this is an interface
        """
        pass

    def add_context(self, prefix):
        """
        Write dtr-specific context for JSON-LD
        The dtr-specific information is provided by the dtr

        :param prefix: A URL prefix
        :return: not implemented, this is an interface
        """
        pass

    def add_dt_type(self, identifier):
        """
        Write schema type for JSON-LD

        :param identifier: a schema identifier
        :return: not implemented, this is an interface
        """
        pass

    def add_dtp_type(self, identifier):
        """
        Write property type for JSON-LD

        :param identifier: a property identifier
        :return: not implemented, this is an interface
        """
        pass

    def add_df_constants(self):
        """
        Write dataframe constants for JSON-LD

        :return: not implemented, this is an interface
        """
        pass


class Epic:
    """
    Class representing the ePIC dtr
    """

    def get_schema_info(self, datatype_id):
        """
        Obtain information from an ePIC datatype

        :param datatype_id: an ePIC datatype identifier
        :return: extracted information from the ePIC datatype
        """
        static = from_static(datatype_id)
        if static is None:
            schema_info = extract_epic(datatype_id)
        else:
            schema_info = static
        return schema_info

    def add_context(self, prefix):
        """
        Write ePIC-specific context for JSON-LD
        The ePIC-specific information is provided by the ePIC dtr

        :param prefix: a URL prefix
        :return: context to include in JSON-LD
        """
        context_info = {
            "doi": prefix,
            "columns": prefix + "0424f6e7026fa4bc2c4a#columns",
            "col_number":  prefix + "65ba00e95e60fb8971e6#number",
            "col_titles":  prefix + "65ba00e95e60fb8971e6#titles",
            "rows":  prefix + "0424f6e7026fa4bc2c4a#rows",
            "row_number":  prefix + "9bf7a8e8909bfd491b38#number",
            "row_titles":  prefix + "9bf7a8e8909bfd491b38#titles",
            "cells":  prefix + "9bf7a8e8909bfd491b38#cells",
            "column":  prefix + "4607bc7c42ac8db29bfc#column",
            "value":  prefix + "4607bc7c42ac8db29bfc#value",
            "tab_label": prefix + "0424f6e7026fa4bc2c4a#label"}
        return context_info

    def add_dt_type(self, identifier):
        """
        Write ePIC-specific datatype type for JSON-LD

        :param identifier: an ePIC datatype identifier
        :return: type to include in JSON-LD
        """
        dt_type = "doi:" + identifier
        return dt_type

    def add_dtp_type(self, identifier):
        """
        Write ePIC-specific property type for JSON-LD

        :param identifier: an ePIC property identifier
        :return: property type to include in JSON-LD
        """
        dtp_type = "doi:" + identifier
        return dtp_type

    def add_df_constants(self):
        """
        Write ePIC-specific dataframe constants for JSON-LD

        :return: dataframe constants to include in JSON-LD
        """
        df_constants = {
            "table": "doi:0424f6e7026fa4bc2c4a",
            "column": "doi:65ba00e95e60fb8971e6",
            "row": "doi:9bf7a8e8909bfd491b38",
            "cell": "doi:4607bc7c42ac8db29bfc"}
        return df_constants


class Orkg:
    """
    Class representing the ORKG dtr
    """

    def get_schema_info(self, datatype_id):
        """
        Obtain information from an ORKG template

        :param datatype_id: an ORKG template identifier
        :return: extracted information from the ORKG template
        """
        schema_info = extract_orkg(datatype_id)
        return schema_info

    def add_context(self, prefix):
        """
        Write ORKG-specific context for JSON-LD
        The ORKG-specific information is provided by the ORKG dtr

        :param prefix: a URL prefix
        :return: context to include in JSON-LD
        """
        context_info = {
            "orkgc": prefix + "class/",
            "orkgr": prefix + "resource/",
            "orkgp": prefix + "property/",
            "columns": prefix + "property/" + "CSVW_Columns",
            "col_number": prefix + "property/" + "CSVW_Number",
            "col_titles": prefix + "property/" + "CSVW_Titles",
            "rows": prefix + "property/" + "CSVW_Rows",
            "row_number": prefix + "property/" + "CSVW_Number",
            "row_titles": prefix + "property/" + "CSVW_Titles",
            "cells": prefix + "property/" + "CSVW_Cells",
            "column": prefix + "property/" + "CSVW_Column",
            "value": prefix + "property/" + "CSVW_Value",
            "label": "http://www.w3.org/2000/01/rdf-schema#label",
            "tab_label": "http://www.w3.org/2000/01/rdf-schema#label"}
        return context_info

    def add_dt_type(self, identifier):
        """
        Write ORKG template type for JSON-LD

        :param identifier: an ORKG template identifier
        :return: type to include in JSON-LD
        """
        dt_type = "orkgr:" + identifier
        return dt_type

    def add_dtp_type(self, identifier):
        """
        Write ORKG-specific property type for JSON-LD

        :param identifier: an ORKG property identifier
        :return: property type to include in JSON-LD
        """
        dtp_type = identifier if identifier == "label" else "orkgp:" + identifier
        return dtp_type

    def add_df_constants(self):
        """
        Write ORKG-specific dataframe constants for JSON-LD

        :return: dataframe constants to include in JSON-LD
        """
        df_constants = {
            "table": "orkgc:Table",
            "column": "orkgc:Column",
            "row": "orkgc:Row",
            "cell": "orkgc:Cell"}
        return df_constants



import json
from importlib_resources import files


def from_static(datatype_id):
    """
    Get schema information from static files

    :param datatype_id: a schema identifier
    :return: the requested schema information, or None if not in static
    """
    id = datatype_id.split("/", 4)[4]
    schema_info = None
    for f in files("dtreg.data").iterdir():
        if id in f.stem:
            file_text = f.read_text()
            schema_info = json.loads(file_text)
    return schema_info



import unittest
from helpers_mock.mocking import mocked_request_orkg, mocked_request_epic


class TestRequest(unittest.TestCase):

    def test_obtain_epic(self):
        schema = mocked_request_epic(
            'https://doi.org/21.T11969/fb2e379f820c6f8f9e82?locatt=view:json')
        self.assertEqual(schema["name"], 'integer_in_string')

    def test_obtain_orkg(self):
        template_2 = mocked_request_orkg('https://orkg.org/api/templates/R758316')
        self.assertEqual(template_2["label"], 'dtreg_test_template2')


if __name__ == '__main__':
    unittest.main()



import unittest
from unittest.mock import patch
from helpers_mock.mocking import mocked_request_orkg
from dtreg.load_datatype import load_datatype


class TestLoadDatatype(unittest.TestCase):

    def test_load_nonstatic(self):
        dt = load_datatype("https://doi.org/21.T11969/fb2e379f820c6f8f9e82")
        self.assertEqual(next(iter(dt.__dict__)), 'integer_in_string')

    def test_load_prop_epic(self):
        dt = load_datatype("https://doi.org/21.T11969/31483624b5c80014b6c7")
        props = dt.matrix_size.prop_list
        expected = ['number_of_rows', 'number_of_columns']
        self.assertEqual(props, expected)

    @patch("dtreg.extract_orkg.request_dtr", mocked_request_orkg)
    def test_load_prop_orkg(self):
        dt = load_datatype("https://orkg.org/template/R758316")
        props = dt.dtreg_test_template2.prop_list
        expected = ['property3', 'label']
        self.assertEqual(props, expected)


if __name__ == '__main__':
    unittest.main()



def mocked_request_epic(route):
    mocked_info = None
    if route == "https://doi.org/21.T11969/31483624b5c80014b6c7?locatt=view:json":
        mocked_info = info_epic_1
    elif route == "https://doi.org/21.T11969/fb2e379f820c6f8f9e82?locatt=view:json":
        mocked_info = info_epic_2
    else:
        print("Please check the URL for mocking")
    return mocked_info


def mocked_request_orkg(route):
    mocked_info = None
    if route == "https://orkg.org/api/templates/R758315":
        mocked_info = info_template_1
    elif route == "https://orkg.org/api/templates/R758316":
        mocked_info = info_template_2
    elif route == "https://orkg.org/api/templates/?target_class=C102007":
        mocked_info = info_via_class
    else:
        print("Please check the URL for mocking")
    return mocked_info


info_epic_1 = {'Identifier': '21.T11969/31483624b5c80014b6c7',
               'name': 'Matrix_Size',
               'Schema': {'Type': 'Object',
                          'Properties': [{'Name': 'number_of_rows',
                                          'Type': '21.T11969/fb2e379f820c6f8f9e82',
                                          'Properties': {'Cardinality': '1'}},
                                         {'Name': 'number_of_columns',
                                          'Type': '21.T11969/fb2e379f820c6f8f9e82',
                                          'Properties': {'Cardinality': '1'}}]}}

info_epic_2 = {'Identifier': "21.T11969/fb2e379f820c6f8f9e82",
               'name': 'integer_in_string',
               'Schema': {'Type': 'String',
                          'Properties': [{'Property': 'pattern', 'Value': '^[0-9]+$'}]
                          }}

info_template_1 = {'id': 'R758315',
                   'label': 'dtreg_test_template1',
                   'target_class': {'id': 'C102006'},
                   'properties': [{
                       'min_count': 0,
                       'max_count': None,
                       'path': {'id': 'P160020', 'label': 'property1'},
                       'datatype': {'id': 'String'}},
                       {'min_count': 0,
                        'max_count': None,
                        'path': {'id': 'P160021', 'label': 'property2'},
                        'class': {'id': 'C102007'}}]}

info_template_2 = {'id': 'R758316',
                   'label': 'dtreg_test_template2',
                   'target_class': {'id': 'C102007'},
                   'properties': [{
                       'min_count': 0,
                       'max_count': None,
                       'path': {'id': 'P160024', 'label': 'property3'},
                       'datatype': {'id': 'Integer'}}]}

info_via_class = {'content': [{'id': 'R758316',
                               'label': 'dtreg_test_template2'}]}



def format_string(string):
    """
    Format a string to lowercase, spaces and dashes as underscores

    :param string: a string
    :return: the string formatted in the required way
    """
    return string.lower().replace(" ", "_").replace("-", "_")


def get_prefix(url_string):
    """
    Get a prefix of a URL string

    :param url_string: a URL string
    :return: the prefix string indicating a datatype
    """
    part = url_string.split("/", 4)
    if "orkg.org" in url_string.split("/", 4)[2]:
        prefix = part[0] + "//" + part[2] + "/"
    elif url_string.split("/", 4)[3] == '21.T11969':
        prefix = part[0] + "//" + part[2] + "/" + part[3] + "/"
    return prefix


def specify_cardinality(cardinality_string):
    """
    Write cardinality of an ePIC property as a dictionary

    :param cardinality_string: an ePIC string for cardinality
    :return: a dictionary with "min" and "max" values as integers or None
    """
    card_parts = cardinality_string.split(" - ")
    if len(card_parts) == 1:
        min_max = {
            "min": int(cardinality_string),
            "max": int(cardinality_string)}
    else:
        min_max = {
            "min": int(card_parts[0]),
            "max": None if card_parts[1] == "n" else int(card_parts[1])}
    return min_max


def generate_uid():
    """
    Generate a counting function for assigning identifiers
    :return: the counting function
    """
    i = 0

    def function():
        nonlocal i
        i += 1
        return i
    return function



import unittest
from unittest.mock import patch
from helpers_mock.mocking import mocked_request_epic
from dtreg.extract_epic import extract_epic


@patch("dtreg.extract_epic.request_dtr", mocked_request_epic)
class TestExtractEpic(unittest.TestCase):

    def test_extract_epic(self):
        result = extract_epic("https://doi.org/21.T11969/fb2e379f820c6f8f9e82")
        expected = {'integer_in_string': [[{'dt_name': 'integer_in_string',
                                            'dt_id': 'fb2e379f820c6f8f9e82',
                                            'dt_class': 'String'}],
                                          [{'dtp_name': 'pattern',
                                            'dtp_id': '21.T11969/fb2e379f820c6f8f9e82#pattern',
                                            'dtp_card_min': None,
                                            'dtp_card_max': None,
                                            'dtp_value_type': '^[0-9]+$'}]]}
        self.assertEqual(result, expected)

    def test_extract_epic_props(self):
        schema = extract_epic("https://doi.org/21.T11969/31483624b5c80014b6c7")
        values = schema["matrix_size"][1][0].values()
        expected = "dict_values(['number_of_rows', "\
            "'21.T11969/31483624b5c80014b6c7#number_of_rows', 1, 1, "\
            "'21.T11969/fb2e379f820c6f8f9e82'])"
        self.assertEqual(str(values), expected)


if __name__ == '__main__':
    unittest.main()



import unittest
from dtreg.helpers import format_string, get_prefix, generate_uid, specify_cardinality


class TestHelpers(unittest.TestCase):

    def test_format_string(self):
        self.assertEqual(format_string("a-B c"), "a_b_c")

    def test_prefix_epic(self):
        self.assertEqual(get_prefix("https://doi.org/21.T11969/74bc7748b8cd520908bc"),
                         "https://doi.org/21.T11969/")

    def test_prefix_orkg(self):
        self.assertEqual(get_prefix("https://incubating.orkg.org/template/R855534"),
                         "https://incubating.orkg.org/")

    def test_specify_cardinality_one(self):
        self.assertEqual(specify_cardinality("1"), {'min': 1, 'max': 1})

    def test_specify_cardinality_two(self):
        self.assertEqual(specify_cardinality("0 - 1"), {'min': 0, 'max': 1})

    def test_uid(self):
        self.assertEqual(str(type(generate_uid())), "<class 'function'>")

    def test_specify_cardinality_n(self):
        self.assertEqual(specify_cardinality("1 - n"), {'min': 1, 'max': None})


if __name__ == '__main__':
    unittest.main()



from .helpers import generate_uid
import pandas as pd
import json
from inspect import isfunction

constants = None
uid = None


def to_jsonld(instance):
    """
    Write an instance in JSON-LD format

    :param instance: an instance of a schema-related class
    :return: JSON string in JSON-LD format
    """
    result_all = {}
    global uid
    uid = generate_uid()
    global constants
    constants = instance.add_df_constants()

    def write_info(instance):
        result = {
            "@id": "_:n" + str(uid()),
            "@type": instance.add_dt_type(instance.dt_id)}
        for field in instance.prop_list:
            instance_field = getattr(instance, field)
            prop_id = next(item for item in instance.prop_info if item["dtp_name"] == field)[
                "dtp_id"]
            prop_type = instance.add_dtp_type(prop_id)
            if instance_field is None or instance_field is []:
                pass
            elif isfunction(instance_field):
                print("Input in " + field + " should not be a function")
            elif isinstance(instance_field, list) and hasattr(instance_field[0], "prop_list"):
                result[prop_type] = list(map(write_info, instance_field))
            elif hasattr(instance_field, "prop_list"):
                result[prop_type] = write_info(instance_field)
            else:
                result[prop_type] = differ_input(instance_field)
        return result
    result_all = write_info(instance)
    result_all["@context"] = instance.add_context(instance.prefix)
    result_json = json.dumps(result_all, indent=2)
    return result_json


def differ_input(input):
    """
    Differentiate input for further use by to_jsonld function

    :param input: an object to be differentiated as a dataframe or another type
    :return: the result of calling the df_structure function, or the input unchanged
    """
    if isinstance(input, pd.DataFrame):
        output = df_structure(input)
    else:
        output = input
    return output


def df_structure(df):
    """
    Prepare a dataframe for to_jsonld function

    :param df: a dataframe
    :return: a dictionary to be used by to_jsonld function
    """
    global uid
    global constants
    result = {}
    result["@type"] = constants["table"]
    result["tab_label"] = df.name if hasattr(df, "name") else "Table"
    column_ids = []
    result["columns"] = []
    for i, col in enumerate(df.columns):
        column = {
            "@type": constants["column"],
            "col_number": i + 1,
            "col_titles": col,
            "@id": "_:n" + str(uid())
        }
        column_ids.append(column["@id"])
        result["columns"].append(column)
    result["rows"] = []
    for i, (title, ro) in enumerate(df.iterrows()):
        row = {
            "@type": constants["row"],
            "row_number": i + 1,
            "row_titles": str(title),
            "@id": "_:n" + str(uid()),
            "cells": []
        }
        for j, cel_val in enumerate(ro):
            row["cells"].append({
                "@type": constants["cell"],
                "@id": "_:n" + str(uid()),
                "value": str(cel_val) if not pd.isna(cel_val) else None,
                "column": column_ids[j]
            })
        result["rows"].append(row)
    result["@id"] = "_:n" + str(uid())
    return result



import requests


def request_dtr(route):
    """
    Request an API of a datatype registry to get information about a schema
    :param route: a path for requesting a dtr API
    :return: requested information about the schema
    """
    r = requests.get(route)
    return r.json()



import unittest

from dtreg.dtr_interface import select_dtr


class TestDataTypeReg(unittest.TestCase):

    def test_dtr_epic(self):
        dtr_epic = select_dtr("https://doi.org/21.T11969/1ea0e148d9bbe08335cd")
        self.assertEqual(str(dtr_epic), "<class 'dtreg.dtr_interface.Epic'>")

    def test_dtr_orkg(self):
        dtr_orkg = select_dtr("https://incubating.orkg.org/template/R937648")
        self.assertEqual(str(dtr_orkg), "<class 'dtreg.dtr_interface.Orkg'>")

    def test_no_dtr(self):
        select_dtr("https://doi.org/22.B34567/1ea0e148d9bbe08335cd")
        self.assertRaisesRegex(
            ValueError,
            "SystemExit: Please check whether the schema belongs to the ePIC or the ORKG dtr")


if __name__ == '__main__':
    unittest.main()



from .dtr_interface import select_dtr
from .helpers import get_prefix
from types import SimpleNamespace


def load_datatype(datatype_id):
    """
    Load classes for a schema with the known identifier

    :param datatype_id: a schema identifier
    :return: a list of schemata as SimpleNamespace objects
    """
    schemata_dict = write_classes(datatype_id)
    schemata = SimpleNamespace(**schemata_dict)
    return schemata


def write_classes(datatype_id):
    """
    Write classes for a specified schema

    :param datatype_id: a schema identifier
    :return: a list of classes
    """
    datypreg = select_dtr(datatype_id)
    schema_info = datypreg().get_schema_info(datatype_id)
    prefix = get_prefix(datatype_id)
    objects = {}
    for key in schema_info.keys():
        dt_name = schema_info[key][0][0]["dt_name"]
        prop_list = []
        if len(schema_info[key][1]) != 0:
            for props in schema_info[key][1]:
                prop_list.append(props["dtp_name"])

        def __init__(self, *args, **kwargs):
            for dtp_name in self.prop_list:
                setattr(self, dtp_name, kwargs.get(dtp_name))
        class_object = type(dt_name,
                            (datypreg,),
                            {"prefix": prefix,
                                "dt_name": dt_name,
                                "dt_id": schema_info[key][0][0]["dt_id"],
                                "dt_class": schema_info[key][0][0]["dt_class"],
                                "prop_list": prop_list,
                                "prop_info": schema_info[key][1],
                                "__init__": __init__})
        objects.update({dt_name: class_object})
    return objects
